﻿namespace UltraGridLoadOnDemand
{
    using System;
    using System.Collections.Generic;
    using System.Text;
    using System.Windows.Forms;
    using Infragistics.Win.UltraWinDataSource;
    using Infragistics.Win.UltraWinGrid;

    public partial class Form1 : Form
    {
        private const string PrimeFactors = "Prime Factors";
        private const string PrimeNumber = "Prime Number";

        private Random random = new Random();

        public Form1()
        {
            this.InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // Set the key of the root band to "Number". This step is optional. We
            // are setting the key here so that later on we can refer to it using the
            // key.
            this.ultraDataSource1.Band.Key = "Number";

            // Add four columns to the root band.
            this.ultraDataSource1.Band.Columns.Add("Decimal", typeof(int));
            this.ultraDataSource1.Band.Columns.Add("Octadecimal", typeof(string));
            this.ultraDataSource1.Band.Columns.Add("Hexadecimal", typeof(string));
            this.ultraDataSource1.Band.Columns.Add("Binary", typeof(string));

            // Create a child band keyed ChildBand1 and add a column to it named
            // "Prime Factors".
            UltraDataBand childBand = this.ultraDataSource1.Band.ChildBands.Add("ChildBand1");
            childBand.Columns.Add(PrimeFactors, typeof(string));

            // Disallow the user to modify the contents.
            this.ultraDataSource1.ReadOnly = true;

            // Set the row count to be 80'000.
            this.ultraDataSource1.Rows.SetCount(80000);

            // Set the LoadStyle to LoadOnDemand for virtual loading. With LoadStyle set to
            // LoadOnDemand, UltraGrid will not create all of the rows at once. It will create
            // rows when they are needed (which typically is when they are scrolled into view).
            this.ultraGrid1.DisplayLayout.LoadStyle = LoadStyle.LoadOnDemand;

            // Set grid's data source to the ultra data source that we just setup.
            this.ultraGrid1.DataSource = this.ultraDataSource1;
        }

        private void UltraDataSource1_InitializeRowsCollection(object sender, Infragistics.Win.UltraWinDataSource.InitializeRowsCollectionEventArgs e)
        {
            if("ChildBand1" == e.Rows.Band.Key)
            {
                // Use the parent row's Index as the integer value for finding the factors.
                int number = e.Rows.ParentRow.Index;

                int[] factors = this.GetFactors(number);

                // Set the row count to be the same as the number of factors. Each row
                // will display a single factor.
                e.Rows.SetCount(factors.Length);

                // Store the prime numbers in the tag of the rows collection so we can retrieve
                // it later in the CellDataRequested event.
                e.Rows.Tag = factors;
            }
        }

        private void UltraDataSource1_CellDataRequested(object sender, CellDataRequestedEventArgs e)
        {
            // Set CacheData to false to prevent the UltraDataSource from caching this value.
            e.CacheData = false;

            if(null == e.Row.Band.ParentBand)
            {
                // For the root band use the row's Index as the integer value and display it
                // in the column in the appropriate format.

                switch(e.Column.Key)
                {
                    case "Decimal":
                        e.Data = e.Row.Index;
                        break;
                    case "Octadecimal":
                        e.Data = this.ToBase(e.Row.Index, 8, "o");
                        break;
                    case "Hexadecimal":
                        e.Data = this.ToBase(e.Row.Index, 16, "h");
                        break;
                    case "Binary":
                        e.Data = this.ToBase(e.Row.Index, 2, "b");
                        break;
                }
            }
            else if("ChildBand1" == e.Row.Band.Key)
            {
                // For the child band, which has a single column named "Prime Factors",
                // display the prime factor based on the child row's index. We stored
                // all the prime factors in the tag in InitializeRowsCollection event
                // which gets fired when the child rows collection is created. Look
                // in there for more info.

                if(PrimeFactors == e.Column.Key)
                {
                    int[] factors = (int[])e.Row.ParentCollection.Tag;
                    if(1 == factors.Length)
                        e.Data = PrimeNumber;
                    else
                        e.Data = factors[e.Row.Index];
                }
            }
        }

        // Converts number n to a string representation in base b.
        private string ToBase(int n, int b, string basePostFix)
        {
            const string digits = "0123456789ABCDEF";

            if(b < 1 || b > digits.Length)
                throw new ArgumentOutOfRangeException("base");

            StringBuilder sb = new StringBuilder();

            if(n < 0)
            {
                sb.Append("-");
                n = -n;
            }

            do
            {
                sb.Append(digits[n % b]);
                n = n / b;
            }
            while(0 != n);

            // Reverse the string.
            int i = 0, j = sb.Length - 1;
            while(i < j)
            {
                char tmp = sb[i];
                sb[i] = sb[j];
                sb[j] = tmp;
                i++;
                j--;
            }

            sb.Append(basePostFix);
            return sb.ToString();
        }

        private int[] GetFactors(int number)
        {
            List<int> list = new List<int>();

            int i = 2;
            while(i <= number)
            {
                if(0 == number % i)
                {
                    list.Add(i);
                    number = number / i;
                }
                else
                {
                    i++;
                }
            }

            return list.ToArray();
        }
    }
}
